package com.yaohun.farmingwar.game;

import com.yaohun.farmingwar.FarmingWar;
import com.yaohun.farmingwar.data.LikeData;
import org.bukkit.*;
import org.bukkit.boss.BarColor;
import org.bukkit.boss.BarStyle;
import org.bukkit.boss.BossBar;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Firework;
import org.bukkit.entity.Player;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.FireworkMeta;
import org.bukkit.plugin.Plugin;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;
import org.bukkit.util.Vector;
import tools.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

public class Game {

    private Region region;
    private World world;
    private BossBar bossBar1;
    private BossBar bossBar2;
    private BossBar bossBar3;
    private boolean started = false;
    private boolean checking = false;
    private int completeAmount;
    private int completeGoal;
    private List<BukkitTask> tasks = new ArrayList<>();
    private List<Entity> entities = new ArrayList<>();
    public Game(){
        this.completeGoal = 15;
        this.world = Bukkit.getWorld("world");
        this.region = new Region(world,new Point(-224,73,246),new Point(-204,73,266));
        bossBar1 = Bukkit.createBossBar("今日挑战进度", BarColor.WHITE, BarStyle.SEGMENTED_10);
        bossBar2 = Bukkit.createBossBar("补种进度", BarColor.GREEN, BarStyle.SEGMENTED_20);
        bossBar3 = Bukkit.createBossBar("点赞累积数量", BarColor.WHITE, BarStyle.SEGMENTED_10);
    }

    private void updateCompleteGoalBossBar() {
        double d = (double) completeAmount / (double) completeGoal;
        String bossTitle = "§6今日挑战进度: §f"+completeAmount+"/"+completeGoal;
        bossBar1.setTitle(bossTitle);
        BossBarUtil.setBarColor(bossBar1,d);
        BossBarUtil.setBarProgress(bossBar1,d);
    }

    private void updateCompleteWheatGoalBossBar() {
        int cNow = GameSite.getCompleteWheatAmount();
        int cMax = GameSite.getCompleteWheatGoal();
        double d = (double) cNow / (double) cMax;
        double d2 = MathUtil.round(d * 100, 2);
        String bossTitle = "§6小麦生长: §f"+cNow+"/"+cMax+" §b("+d2+"%)";
        bossBar2.setTitle(bossTitle);
        BossBarUtil.setBarColor(bossBar2,d);
        BossBarUtil.setBarProgress(bossBar2,d);
    }
    public void updateDianZanBossBar() {
        int dz = 0;
        int needDz = 300;
        String eventName = "";
        HashMap<String, LikeData> likeDataMap = FarmingWar.getGameManager().getLikeDataMap();
        for (String key : likeDataMap.keySet()){
            LikeData likeData = likeDataMap.get(key);
            dz = likeData.getNowAmount();
            needDz = likeData.getNeedAmount();
            eventName = likeData.getEffectKey();
            break;
        }
        int dz_need = needDz;
        double dz1 = (double) dz / (double) dz_need;
        String bossTitle = "§6"+eventName+"(戳一戳): §d"+dz+"§f/§c"+dz_need;
        bossBar3.setTitle(bossTitle);
        BossBarUtil.setBarColor(bossBar3,dz1);
        BossBarUtil.setBarProgress(bossBar3,dz1);
    }

    public void start() {
        if (started) {
            return;
        }
        this.started = true;
        this.checking = false;
        initWorld();
        GameUtil.loadSchematics("muban");
        this.completeAmount = 0;
        tasks.add(new BukkitRunnable() {
            @Override
            public void run() {
                updateCompleteGoalBossBar();
                updateCompleteWheatGoalBossBar();
                updateDianZanBossBar();
                if (!checking) {
                    checkSiteDegreeOfCompletion();
                }
            }
        }.runTaskTimer(FarmingWar.inst(), 0L, 10L));
    }
    public void initPlayerData(Player player) {
        player.setArrowCooldown(3600);
        player.setGameMode(GameMode.SURVIVAL);
        player.setAllowFlight(true);
        player.teleport(region.getHub());
        bossBar3.addPlayer(player);
        initPlayerBackpack(player);
    }
    public void initPlayerBackpack(Player player){
        Inventory inv = player.getInventory();
        inv.setItem(0, StackUtil.diamondSword());
        inv.setItem(1, StackUtil.trident());
        inv.setItem(2, StackUtil.quickPotion());
        inv.setItem(3, new ItemStack(Material.WHEAT_SEEDS,32));
        inv.setItem(4, new ItemStack(Material.BONE_MEAL,32));
    }
    private void initWorld() {
        this.world.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
        this.world.setGameRule(GameRule.DO_WEATHER_CYCLE, false);
        this.world.setGameRule(GameRule.DO_MOB_SPAWNING, false);
        this.world.setGameRule(GameRule.DO_MOB_LOOT, false);
        this.world.setGameRule(GameRule.MOB_GRIEFING, false);
        this.world.setGameRule(GameRule.RANDOM_TICK_SPEED, 0);
        this.world.setTime(4500);
        this.world.setDifficulty(Difficulty.NORMAL);
    }

    private int countdown = 0;
    private int countdownTime = 11;
    public void checkSiteDegreeOfCompletion() {
        if(checking){return;}
        int i = GameSite.getCompleteWheatAmount();
        if(i >= GameSite.getCompleteWheatGoal()){
            checking = true;
            countdown = 0;
            BukkitTask task = new BukkitRunnable() {
                @Override
                public void run() {
                    int i = GameSite.getCompleteWheatAmount();
                    // 倒计时期间小麦受到破坏则停止倒计时
                    if(i < GameSite.getCompleteWheatGoal()){
                        tasks.remove(this);
                        checking = false;
                        Bukkit.broadcastMessage("§c[系统]§a小麦受到了破坏,需要进行修复.");
                        for (Player player : Bukkit.getOnlinePlayers()) {
                            player.sendTitle("§c哎呀!", "§6继续加油!");
                            player.playSound(player.getLocation(), "niganma", 1.0F, 1.0F);
                        }
                        cancel();
                        return;
                    }
                    if(countdown >= countdownTime-1){
                        tasks.add(new BukkitRunnable() {
                            private int i = 0;
                            @Override
                            public void run() {
                                if(i >= 20){
                                    cancel();
                                    return;
                                }
                                for (Player player : Bukkit.getOnlinePlayers()){
                                    shoutFirework(player.getLocation().clone().add(RandomUtil.getRandomInt(-10,10),RandomUtil.getRandomInt(3,6),RandomUtil.getRandomInt(-10,10)));
                                }
                                i++;
                            }
                        }.runTaskTimer(FarmingWar.inst(),0L,5L));
                        tasks.add(new BukkitRunnable() {
                            private int i = 0;
                            @Override
                            public void run() {
                                if (i >= 100) {
                                    cancel();
                                    return;
                                }
                                region.getWorld().setTime(region.getWorld().getTime() + 480L);
                                i++;
                            }
                        }.runTaskTimer(FarmingWar.inst(), 0L, 1L));
                        completeAmount++;
                        tasks.remove(this);
                        checking = false;
                        Bukkit.broadcastMessage("§c[系统]§a挑战成功! §b游戏进度+1");
                        for (Player player : Bukkit.getOnlinePlayers()) {
                            player.playSound(player.getLocation(), "duolaameng", 1.0F, 1.0F);
                            player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_LEVELUP, 1.0f, 1.1f);
                        }
                        GameSite.restoreInitialization();
                        cancel();
                        return;
                    }
                    countdownEvent();
                }
            }.runTaskTimer(FarmingWar.inst(),0,20L);
            addTasks(task);
        }
    }
    private void countdownEvent(){
        countdown++;
        int daojishi = countdownTime;
        for (Player player : Bukkit.getOnlinePlayers()) {
            player.sendTitle("§6§l" + (daojishi - countdown), "§6即将挑战成功!");
            player.playSound(player.getLocation(), "daojishi", 1.0f, 1.0f);
        }
    }

    public Region getRegion() {return region;}
    public boolean isStarted() {return started;}
    public List<BukkitTask> getTasks() {
        return tasks;
    }
    public void addTasks(BukkitTask task){
        this.tasks.add(task);
    }

    private void shoutFirework(Location location){
        // 创建烟花对象
        Firework firework = location.getWorld().spawn(location.clone().add(0,1,0), Firework.class);
        FireworkMeta fireworkMeta = firework.getFireworkMeta();
        Color color = Color.YELLOW;
        int rand = RandomUtil.getRandomInt(1,100);
        if(rand >= 75){
            color = Color.RED;
        }else if(rand >= 50){
            color = Color.AQUA;
        }else if(rand >= 25){
            color = Color.GREEN;
        }
        // 设置烟花效果
        FireworkEffect fireworkEffect = FireworkEffect.builder()
                .withColor(color)
                .with(FireworkEffect.Type.BURST)
                .build();
        firework.setVelocity(new Vector(0, -0.2, 0));
        fireworkMeta.addEffect(fireworkEffect);
        fireworkMeta.setPower(0); // 设置烟花强度
        firework.setFireworkMeta(fireworkMeta);
        new BukkitRunnable() {
            public void run() {
                firework.detonate();
                cancel();
            }
        }.runTaskLater( FarmingWar.inst(), 5L );
    }

    public void createRegion(String regionKey, Region region) {
        FileConfiguration config = FarmingWar.inst().getConfig();
        ConfigurationSection section = config.createSection(regionKey);
        section.set("world", region.getWorld().getName());
        Point point1 = region.getMin();
        Point point2 = region.getMax();
        ConfigurationSection section1 = section.createSection("min");
        section1.set("x", point1.getX());
        section1.set("y", point1.getY());
        section1.set("z", point1.getZ());
        ConfigurationSection section2 = section.createSection("max");
        section2.set("x", point2.getX());
        section2.set("y", point2.getY());
        section2.set("z", point2.getZ());
        FarmingWar.inst().saveConfig();
    }
}
